module BitmapCompiler
  module Core
    # Bitmap entity
    class Bitmap
      MIN_WIDTH = MIN_HEIGHT = 1
      MAX_WIDTH = MAX_HEIGHT = 250
      STANDARD_COLOR = 'O'.freeze

      attr_reader :width, :height, :cells

      def initialize(width, height, cells: nil)
        @height = Integer(height)
        @width = Integer(width)
        @cells = cells

        clear if cells.nil?
      end

      def change_pixel(row, column, color)
        return false unless valid_coordinate?(row, column)

        cells[row - 1][column - 1] = color
      end

      def clear
        @cells = initialize_matrix
      end

      def initialize_matrix
        Array.new(width) { Array.new(height) { STANDARD_COLOR } }
      end

      def print
        ''.tap do |output|
          (0..cells[0].length - 1).each do |column_index|
            (0..cells.length - 1).each do |row_index|
              output << cells[row_index][column_index]
            end

            output << "\n"
          end
        end
      end

      def valid_vertical_line_coordinates?(row, start_column, end_column)
        valid_row_index?(row) && valid_column_index?(start_column) &&
          valid_column_index?(end_column) && start_column <= end_column
      end

      def valid_horizontal_line_coordinates?(column, start_row, end_row)
        valid_row_index?(end_row) && valid_row_index?(start_row) &&
          valid_column_index?(column) && start_row <= end_row
      end

      def valid_coordinate?(row, column)
        valid_row_index?(row) && valid_column_index?(column)
      end

      def valid_column_index?(column)
        column.between?(1, height)
      end

      def valid_row_index?(row)
        row.between?(1, width)
      end
    end
  end
end
