module BitmapCompiler
  module Core
    # BitmapActions shares role behavior
    module BitmapActions
      def change_color(row:, column:, color: Bitmap::STANDARD_COLOR)
        return image_not_found unless bitmap

        if bitmap.change_pixel(row, column, color)
          return_output
        else
          raise BitmapCompiler::Core::Errors::InvalidPixelError
        end
      end

      def clear
        return image_not_found unless bitmap

        if bitmap.clear
          return_output
        else
          raise BitmapCompiler::Core::Errors::ImageNotClearedError
        end
      end

      def create_bitmap(args)
        if args[:width].between?(Bitmap::MIN_WIDTH, Bitmap::MAX_WIDTH) &&
           args[:height].between?(Bitmap::MIN_HEIGHT, Bitmap::MAX_HEIGHT)
          return_output(bitmap: Bitmap.new(args[:width], args[:height]))
        else
          raise BitmapCompiler::Core::Errors::InvalidDimensionsError
        end
      end

      def print
        return image_not_found unless bitmap

        return_output(message: bitmap.print)
      end

      def change_horizontal_line(column:, start_row:, end_row:, color: Bitmap::STANDARD_COLOR)
        return image_not_found unless bitmap

        if bitmap.valid_horizontal_line_coordinates?(column, start_row, end_row)
          (start_row..end_row).each do |row_index|
            bitmap.change_pixel(row_index, column, color)
          end

          return_output
        else
          raise Errors::InvalidCoordinatesError
        end
      end

      def change_vertical_line(row:, start_column:, end_column:, color: Bitmap::STANDARD_COLOR)
        image_not_found unless bitmap

        if bitmap.valid_vertical_line_coordinates?(row, start_column, end_column)
          (start_column..end_column).each do |column_index|
            bitmap.change_pixel(row, column_index, color)
          end

          return_output
        else
          raise BitmapCompiler::Core::Errors::InvalidCoordinatesError
        end
      end

      def unknown_command
        raise BitmapCompiler::Core::Errors::UnknownCommandError
      end

      private

      def image_not_found
        raise BitmapCompiler::Core::Errors::NoImageFoundError
      end

      def return_output(args = {})
        Output.new(bitmap: args[:bitmap], message: args[:message])
      end
    end
  end
end
