# frozen_string_literal: true

module BitmapCompiler
  module Core
    # Responsibility: execute commands
    class BitmapController
      include BitmapActions

      def initialize(bitmap = nil)
        @bitmap = bitmap
      end

      attr_reader :bitmap, :input

      def execute_command(input)
        @input = input.split

        case input[0]
        when 'I'
          create_bitmap(bitmap_hash)
        when 'C'
          clear
        when 'L'
          change_color(pixel_hash)
        when 'V'
          change_vertical_line(vertical_line_hash)
        when 'H'
          change_horizontal_line(horizontal_line_hash)
        when 'S'
          print
        else
          unknown_command
        end
      end

      private

      def bitmap_hash
        {
          width: Integer(input[1]),
          height: Integer(input[2])
        }
      end

      def horizontal_line_hash
        {
          start_row: Integer(input[1]),
          end_row: Integer(input[2]),
          column: Integer(input[3]),
          color: input[4]
        }
      end

      def pixel_hash
        {
          row: Integer(input[1]),
          column: Integer(input[2]),
          color: input[3]
        }
      end

      def vertical_line_hash
        {
          row: Integer(input[1]),
          start_column: Integer(input[2]),
          end_column: Integer(input[3]),
          color: input[4]
        }
      end
    end
  end
end
