# frozen_string_literal: true

module BitmapCompiler
  module Core
    # Responsibility: process the files
    class Editor
      attr_reader :bitmap

      def initialize(bitmap: nil)
        @bitmap = bitmap
      end

      def run(file)
        raise BitmapCompiler::Core::Errors::IncorrectFileError if file.nil? || !File.exist?(file)

        File.open(file).each { |line| process_line(line.chomp) }
      end

      def process_line(input)
        output = execute(input)

        @bitmap = output.bitmap if output.bitmap

        output.print
      end

      private

      def execute(input)
        bitmap_compiler = BitmapController.new(bitmap)
        bitmap_compiler.execute_command(input)
      end
    end
  end
end
